/**
 * Converts the given dsid to a dsuid
 * @param {String} dSID
 * @returns {String} dSUID
 */
function dSID2dSUID(dSID) {
    try {
        //copy the first 6 bytes (12 characters to the new dsuid
        var dSUID = dSID.slice(0, 12);
        //insert 4 bytes of 0
        dSUID += '00000000';
        //append the remaining 6 bytes (12 chars) of the dsid
        dSUID += dSID.slice(12, 24);
        //finaly add 1 byte of 0
        dSUID += '00';
        return dSUID;
    } catch (e) {
        lWarn.logln('Exception in hue-controller-config::dSID2dSUID ' + e);
        return '';
    }
}

/**
 * Converts the trigger entries from dsid to dsuid
 * @returns {boolean} true if property tree changed, false otherwize
 */
function convert2DSUID() {
    var fMigrate = false;
    try {
        lInfo.logln('convert2DSUID start');
        var iIndex, jIndex;
        var dSID, dSUID;
    	for (var activityTypeName in ActivityUpdateTypes)
        {
            var activityType = ActivityUpdateTypes[activityTypeName];

            var baseActivityNode = Property.getNode(activityType);
            if (baseActivityNode != null) {
                var activityChildren = baseActivityNode.getChildren();
                for (iIndex = 0; iIndex < activityChildren.length; iIndex++) {
                    var triggerNode = activityChildren[iIndex].getChild('triggers');
                    if (triggerNode!=null) {
	                    var triggerArray = triggerNode.getChildren();
	                    for (jIndex = 0; jIndex < triggerArray.length; jIndex++) {
	                        switch(triggerArray[jIndex].getChild('type').getValue()) {
	                            case 'device-msg': //intend fallthrough
	                            case 'device-binary-input': //intend fallthrough
	                            case 'device-scene': //intend fallthrough
	                            case 'device-sensor': {
	                                if (triggerArray[jIndex].getChild('dsid') != null) {
	                                    lInfo.logln('convert');    
	                                    dSID = triggerArray[jIndex].getChild('dsid').getValue();
	                                    dSUID = dSID2dSUID(dSID);
	                                    Property.setProperty(triggerArray[jIndex].getPath() + '/dsuid', dSUID);
	                                    Property.setFlag(triggerArray[jIndex].getPath() + '/dsuid', 'ARCHIVE', true);
	                                    triggerArray[jIndex].removeChild('dsid');
	                                    fMigrate = true;
	                                }
	                            }
	                            break;
	                            case 'state-change': {
	                                    var triggerName = triggerArray[jIndex].getChild('name').getValue();
	                                    var nameArray = triggerName.split('.');
	                                    if ((nameArray[0] === 'dev') && (nameArray[1].length === 24)) {
	                                        var dsuid = dSID2dSUID(nameArray[1]);
	                                        triggerName = '';
	                                        triggerName = triggerName.concat(nameArray[0],'.', dsuid, '.', nameArray[2]);
	                                        Property.setProperty(triggerArray[jIndex].getPath() + '/name', triggerName);
	                                        Property.setFlag(triggerArray[jIndex].getPath() + '/name', 'ARCHIVE', true);
	                                        fMigrate=true;
	                                    }
	                                }
	                            break;
	                            default:{
	                                Property.setProperty(triggerArray[jIndex].getPath() + '/dsuid', -1);
	                                Property.setFlag(triggerArray[jIndex].getPath() + '/dsuid', 'ARCHIVE', true);
	                                triggerArray[jIndex].removeChild('dsid');
	                                fMigrate = true;
	                            }
	                        }
	                    }
                    }
                }
            }
        }
    } catch(e) {
        lWarn.logln('Exception in hue-controller-config::convert2DSUID: ' + e);
    }
    return fMigrate;
}

/**
 * Compares version strings
 * @param {string} strV1 version string 1 to compare 
 * @param {string} strV2 version string 1 to compare
 * @returns {Number|undefined} return   number < 0  if strV1 < strV2
 *                                          0       if strV1 = strV2
 *                                      number >0   if strV1 < strV2
 */
function compVersions(strV1, strV2) {
    try {
        var nRes = 0;
        var parts1 = strV1.split('.');
        var parts2 = strV2.split('.');
        var nLen = Math.max(parts1.length, parts2.length);

        for (var i = 0; i < nLen; i++) {
          var nP1 = (i < parts1.length) ? parseInt(parts1[i], 10) : 0;
          var nP2 = (i < parts2.length) ? parseInt(parts2[i], 10) : 0;

          if (isNaN(nP1)) { nP1 = 0; }
          if (isNaN(nP2)) { nP2 = 0; }

          if (nP1 !== nP2) {
            nRes = (nP1 > nP2) ? 1 : -1;
            break;
          }
        }
        return nRes;
    } catch(e) {
        lWarn.logln('Exception in hue-controller-config:compVersions ' + e);
        return 0;       
    }
}

/**
 * convert property tree entries if needed
 * @returns {undefined}
 */
function convertPropertyTreeEntries()
{
    var newVersion = '1.0.10';
    var fMigrate = false;
    try {      
        var propVersion = Property.getNode('version');
        var currentVersion = (propVersion != null) ? propVersion.getValue() : '0.0.0';
        lInfo.logln('check if conversions from version "' + currentVersion + '" to version "' + newVersion + '" are needed');
        fMigrate = convert2DSUID() || fMigrate;  
        /*if (compVersions(currentVersion, newVersion) < 0) {
            fMigrate = convert2DSUID() || fMigrate;   
        } else {
            lInfo.logln('no conversion needed');    
        }*/
        Property.setProperty('version', newVersion);
        Property.setFlag('version', 'ARCHIVE', true);
        lInfo.logln("hue controller running (Version " + newVersion + ")");
    } catch (e) {
        lWarn.logln('Exception in hue-controller-config::convertPropertyTreeEntries ' + e);
    }
    Property.store();
}

/* Visits each activity node and registers the corresponding trigger.
 * This is necessary after starting the DSS to make sure the triggers are configured properly. */
function updateActivityTriggers()
{
	for (var activityTypeName in ActivityTypes)
	{
		var activityType = ActivityTypes[activityTypeName];

		var baseActivityNode = Property.getNode(activityType);
		if (!baseActivityNode)
		{
			lInfo.logln("updateActivityTriggers: found no subnode for activities of type " + activityType);
			continue;
		}

		var children = baseActivityNode.getChildren();
		lInfo.logln("updateActivityTriggers: found " + String(children.length) + " activities of type " + activityType);

		for (var i = 0; i < children.length; ++i)
		{
			var child = children[i];
			var activityID = child.getName();
			lInfo.logln("updateActivityTriggers: registering trigger for activity of type " + activityType + " with ID " + activityID);

			registerTrigger(child.getPath(), "hueController.handleTrigger", { });
		}
	}

//	Property.setProperty("preset/predefined/dimming/placeholder", "placeholder");
	Property.setProperty("preset/predefined/off/placeholder", "placeholder");
//	registerTrigger(Property.getNode("preset/predefined/dimming").getPath(), "hueController.handleDimming", { });
	registerTrigger(Property.getNode("preset/predefined/off").getPath(), "hueController.handleOff", { });
}




/* main event handling */


if (raisedEvent.name == 'running')
{
	Property.load();
	try
	{
		convertPropertyTreeEntries();
        updateActivityTriggers();
	}
	catch (err)
	{
		lWarn.logln('Error while updating triggers: ' + String(err));
	}
}


